from swarmauri_base.loggers.LoggerBase import LoggerBase
from swarmauri_base import register_type


@register_type()
class Logger(LoggerBase):
    """Concrete implementation of LoggerBase"""

    pass
